/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.filter.odf.odp.dom;

import org.apache.xml.serializer.SerializationHandler;
import org.json.JSONException;
import org.json.JSONObject;
import org.odftoolkit.odfdom.IElementWriter;
import org.xml.sax.SAXException;
import com.openexchange.office.filter.core.DLList;
import com.openexchange.office.filter.core.DLNode;
import com.openexchange.office.filter.core.INodeAccessor;
import com.openexchange.office.filter.odf.AttributesImpl;
import com.openexchange.office.filter.odf.Namespaces;
import com.openexchange.office.filter.odf.OdfOperationDoc;
import com.openexchange.office.filter.odf.OpAttrs;
import com.openexchange.office.filter.odf.odt.dom.Annotation;
import com.openexchange.office.filter.odf.styles.StyleBase;
import com.openexchange.office.filter.odf.styles.StyleFamily;
import com.openexchange.office.filter.odf.styles.StyleManager;

public abstract class Page implements IElementWriter, INodeAccessor {

    protected AttributesImpl attributes;

    private DLList<Object> content;
    private DLList<Annotation> annotations;
    PresentationNotes presentationNotes;

    public Page(AttributesImpl attributes) {
	    this.attributes = attributes;
	}

	@Override
	public DLList<Object> getContent() {
        if (content == null) {
            content = new DLList<Object>();
        }
		return content;
	}

	public DLList<Annotation> getAnnotations(boolean forceCreate) {
	    if(annotations==null && forceCreate) {
	        annotations = new DLList<Annotation>();
	    }
	    return annotations;
	}

	abstract public String getName();

	abstract public void setName(String name);

	final public String getStyle() {
	    return attributes.getValue("draw:style-name");
	}

	final public void setStyle(String styleName) {
	    attributes.setValue(Namespaces.DRAW, "style-name", "draw:style-name", styleName);
	}

    @Override
	public abstract void writeObject(SerializationHandler output) throws SAXException;

    protected void writeComments(SerializationHandler output)
        throws SAXException {

        if(annotations!=null) {
            DLNode<Annotation> annotationNode = annotations.getFirstNode();
            while(annotationNode!=null) {
                annotationNode.getData().writeObject(output);
                annotationNode = annotationNode.getNext();
            }
        }
    }

    @SuppressWarnings("unused")
    public void applyAttrsFromJSON(OdfOperationDoc operationDocument, boolean contentAutoStyle, JSONObject attrs) throws JSONException, SAXException {
        setStyle(operationDocument.getDocument().getStyleManager().createStyle(StyleFamily.DRAWING_PAGE, getStyle(), contentAutoStyle, attrs));
    }

    public void createAttrs(OdfOperationDoc operationDocument, boolean contentAutoStyle, OpAttrs attrs) {
        final String style = getStyle();
        if(style!=null&&!style.isEmpty()) {
            final StyleManager styleManager = operationDocument.getDocument().getStyleManager();
            final StyleBase styleBase = styleManager.getStyle(style, StyleFamily.DRAWING_PAGE, contentAutoStyle);
            if(styleBase!=null) {
                styleBase.createAttrs(styleManager, attrs);
            }
        }
    }


}
