/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.filter.odf.styles;

import org.xml.sax.Attributes;
import com.openexchange.office.filter.odf.AttributesImpl;
import com.openexchange.office.filter.odf.ElementNSWriter;
import com.openexchange.office.filter.odf.SaxContextHandler;
import com.openexchange.office.filter.odf.UnknownContentHandler;

public class AutomaticStylesHandler extends SaxContextHandler {

    private final StyleManager styleManager;
    private final boolean contentStyle;

    public AutomaticStylesHandler(SaxContextHandler parentContext, StyleManager styleManager, boolean contentStyle) {
    	super(parentContext);

    	this.styleManager = styleManager;
    	this.contentStyle = contentStyle;
    }

    @Override
    public SaxContextHandler startElement(Attributes attributes, String uri, String localName, String qName) {

    	final AttributesImpl attributesImpl = new AttributesImpl(attributes);
    	String name = StylesHandler.removeNameFromAttributes(attributesImpl);
    	if(name==null) {
    		// default styles does not have a name, we will check this...
    		if(qName.equals("style:default-style")) {
    			name = "_default";
    		}
    	}
    	if(name!=null) {
	    	switch(qName) {
				case "style:default-style" : {
					final SaxContextHandler styleHandler = StylesHandler.getStyleHandler(this, name, styleManager, attributesImpl, true, true, contentStyle);
					if(styleHandler!=null) {
						return styleHandler;
					}
					break;
				}
				case "style:style" : {
					final SaxContextHandler styleHandler = StylesHandler.getStyleHandler(this, name, styleManager, attributesImpl, false, true, contentStyle);
					if(styleHandler!=null) {
						return styleHandler;
					}
					break;
				}
				case "style:page-layout": {
					return new StylePageLayoutHandler(this, new StylePageLayout(name, attributesImpl, false, true, contentStyle), styleManager);
				}
				case "number:number-style": {
		    		return new NumberNumberStyleHandler(this, new NumberNumberStyle(name, attributesImpl, true, contentStyle), styleManager);
				}
				case "number:boolean-style": {
		    		return new NumberBooleanStyleHandler(this, new NumberBooleanStyle(name, attributesImpl, true, contentStyle), styleManager);
				}
				case "number:currency-style" : {
		    		return new NumberCurrencyStyleHandler(this, new NumberCurrencyStyle(name, attributesImpl, true, contentStyle), styleManager);
				}
				case "number:date-style" : {
		    		return new NumberDateStyleHandler(this, new NumberDateStyle(name, attributesImpl, true, contentStyle), styleManager);
				}
				case "number:percentage-style" : {
		    		return new NumberPercentageStyleHandler(this, new NumberPercentageStyle(name, attributesImpl, true, contentStyle), styleManager);
				}
				case "number:text-style" : {
		    		return new NumberTextStyleHandler(this, new NumberTextStyle(name, attributesImpl, true, contentStyle), styleManager);
				}
				case "number:time-style" : {
		    		return new NumberTimeStyleHandler(this, new NumberTimeStyle(name, attributesImpl, true, contentStyle), styleManager);
				}
				case "text:list-style" : {
		    		return new TextListStyleHandler(this, new TextListStyle(name, attributesImpl, true, contentStyle), styleManager);
				}
			}
    	}
    	final ElementNSWriter element = new ElementNSWriter(getFileDom(), attributes, uri, qName);
    	styleManager.getAutomaticStyles(contentStyle).getContent().add(element);
		return new UnknownContentHandler(this, element);
    }
}
