/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.spellchecker.impl.client;

import org.springframework.stereotype.Service;
import com.openexchange.config.DefaultInterests;
import com.openexchange.config.Interests;
import com.openexchange.office.tools.annotation.RegisteredService;
import com.openexchange.rest.client.httpclient.DefaultHttpClientConfigProvider;
import com.openexchange.rest.client.httpclient.HttpBasicConfig;
import com.openexchange.rest.client.httpclient.SpecificHttpClientConfigProvider;

/**
 * {@link SpellCheckerHttpConfigProvider}
 *
 * @author <a href="mailto:kai.ahrens@open-xchange.com">Kai Ahrens</a>
 * @since v7.10.4
 */
@Service
@RegisteredService(registeredClass=SpecificHttpClientConfigProvider.class)
public class SpellCheckerHttpConfigProvider extends DefaultHttpClientConfigProvider {

    public static final String HTTP_CLIENT_ID = "spellchecker";
    public static final String HTTP_CLIENT_USER_AGENT = "Open-Xchange SpellChecker";

    final private static int CONNECT_TIMEOUT_MILLIS = 30000;
    final private static int READ_TIMEOUT_MILLIS = 30000;
    final private static int TOTAL_CONNECTIONS = 24;
    final private static int TOTAL_CONNECTIONS_PER_ROUTE = 12;

    /**
     * Initializes a new {@link SpellCheckerHttpConfigProvider}.
     */
    public SpellCheckerHttpConfigProvider() {
        super(HTTP_CLIENT_ID, HTTP_CLIENT_USER_AGENT);
    }

    @Override
    public Interests getAdditionalInterests() {
        return DefaultInterests.builder().propertiesOfInterest("com.openexchange.spellchecker.*").build();
    }

    @Override
    public HttpBasicConfig configureHttpBasicConfig(HttpBasicConfig config) {
        config.setConnectTimeout(CONNECT_TIMEOUT_MILLIS);
        config.setSocketReadTimeout(READ_TIMEOUT_MILLIS);
        config.setMaxTotalConnections(TOTAL_CONNECTIONS);
        config.setMaxConnectionsPerRoute(TOTAL_CONNECTIONS_PER_ROUTE);

        return config;
    }

    /**
     * @return
     */
    public static int getConnectTimeoutMillis() {
        return CONNECT_TIMEOUT_MILLIS;
    }

    /**
     * @return
     */
    public static int getSocketReadTimeoutMillis() {
        return READ_TIMEOUT_MILLIS;
    }
}