/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
package com.openexchange.office.tools.common.user;

import org.apache.commons.lang3.StringUtils;
import com.openexchange.user.User;

/**
 * Helper functions regarding the User class
 * {@link UserHelper}
 *
 * @author <a href="mailto:carsten.driesner@open-xchange.com">Carsten Driesner</a>
 * @since v7.8.0
 */
public class UserHelper {

    //-------------------------------------------------------------------------
    public static String getFullName(final User user) {
        String fullName = null;

        if (null != user) {
            final String givenName = user.getGivenName();
            final String surName = user.getSurname();
            boolean hasGivenName = StringUtils.isNotBlank(givenName);
            boolean hasSurName = StringUtils.isNotBlank(surName);

            if (hasSurName) {
                final StringBuilder tmp = new StringBuilder(64);
                tmp.append(surName);
                if (hasGivenName) {
                    tmp.append(", ");
                    tmp.append(givenName);
                }
                fullName = tmp.toString();
            } else if(hasGivenName){
                fullName = givenName;
            }
        }

        return fullName;
    }

    //-------------------------------------------------------------------------
    public static String getLangCode(final User user) {
        String sLangCode = null;

        if (null != user) {
            sLangCode =  mapUserLanguageToLangCode(user.getPreferredLanguage());
        }

        return sLangCode;
    }

    //-------------------------------------------------------------------------
    /**
     * Maps the user language provided by a User object to a ISO compliant
     * language code. E.g. en_US => en-US
     *
     * @param userLanguage
     *  The user language as provided by the User methods getPreferredLanguage
     *
     * @return
     *  An ISO compliant language code or null if language code couldn't be
     *  parsed correctly.
     */
    static public String mapUserLanguageToLangCode(String userLanguage) {
        return (null != userLanguage) ? (userLanguage.indexOf('_') >= 0 ? userLanguage.replace('_', '-') : userLanguage) : null;
    }

    private UserHelper() {}
}
